@@ -1,8 +1,14 @@
 # encoding: utf-8
 from __future__ import unicode_literals
 
-import sys
-PY_3_OR_HIGHER = sys.version_info >= (3, 0)
+from .util import PY_3_OR_HIGHER
+
+try:
+    import ssl
+except ImportError:
+    _HAVE_SSL = False
+else:
+    _HAVE_SSL = True
 
 if PY_3_OR_HIGHER:
     import urllib.request as urllib_request
@@ -174,7 +180,11 @@ class TwitterJSONIter(object):
 
         while True:
             # Decode all the things:
-            data = sock_reader.read()
+            try:
+                data = sock_reader.read()
+            except SSLError:
+                yield Hangup
+                break
             dechunked_data, end_of_stream, decode_error = chunk_decoder.decode(data)
             unicode_data = utf8_decoder.decode(dechunked_data)
             json_data = json_decoder.decode(unicode_data)
@@ -203,9 +213,12 @@ class TwitterJSONIter(object):
                 yield self.timeout_token
 
 
-def handle_stream_response(req, uri, arg_data, block, timeout, heartbeat_timeout):
+def handle_stream_response(req, uri, arg_data, block, timeout, heartbeat_timeout, verify_context=True):
     try:
-        handle = urllib_request.urlopen(req,)
+        context = None
+        if not verify_context and _HAVE_SSL:
+            context = ssl._create_unverified_context()
+        handle = urllib_request.urlopen(req, context=context)
     except urllib_error.HTTPError as e:
         raise TwitterHTTPError(e, uri, 'json', arg_data)
     return iter(TwitterJSONIter(handle, uri, arg_data, block, timeout, heartbeat_timeout))
@@ -227,31 +240,7 @@ class TwitterStream(TwitterCall):
     [public streams](https://dev.twitter.com/docs/streaming-apis/streams/public).
     If you want to use one of the other
     [streaming APIs](https://dev.twitter.com/docs/streaming-apis), specify the URL
-    manually:
-
-    - [Public streams](https://dev.twitter.com/docs/streaming-apis/streams/public): stream.twitter.com
-    - [User streams](https://dev.twitter.com/docs/streaming-apis/streams/user): userstream.twitter.com
-    - [Site streams](https://dev.twitter.com/docs/streaming-apis/streams/site): sitestream.twitter.com
-
-    Note that you require the proper
-    [permissions](https://dev.twitter.com/docs/application-permission-model) to
-    access these streams. E.g. for direct messages your
-    [application](https://dev.twitter.com/apps) needs the "Read, Write & Direct
-    Messages" permission.
-
-    The following example demonstrates how to retrieve all new direct messages
-    from the user stream::
-
-        auth = OAuth(
-            consumer_key='[your consumer key]',
-            consumer_secret='[your consumer secret]',
-            token='[your token]',
-            token_secret='[your token secret]'
-        )
-        twitter_userstream = TwitterStream(auth=auth, domain='userstream.twitter.com')
-        for msg in twitter_userstream.user():
-            if 'direct_message' in msg:
-                print msg['direct_message']['text']
+    manually.
 
     The iterator will yield until the TCP connection breaks. When the
     connection breaks, the iterator yields `{'hangup': True}`, and
@@ -275,16 +264,17 @@ class TwitterStream(TwitterCall):
     """
     def __init__(self, domain="stream.twitter.com", secure=True, auth=None,
                  api_version='1.1', block=True, timeout=None,
-                 heartbeat_timeout=90.0):
+                 heartbeat_timeout=90.0, verify_context=True):
         uriparts = (str(api_version),)
 
         class TwitterStreamCall(TwitterCall):
             def _handle_response(self, req, uri, arg_data, _timeout=None):
                 return handle_stream_response(
                     req, uri, arg_data, block,
-                    _timeout or timeout, heartbeat_timeout)
+                    _timeout or timeout, heartbeat_timeout, verify_context)
 
         TwitterCall.__init__(
             self, auth=auth, format="json", domain=domain,
             callable_cls=TwitterStreamCall,
-            secure=secure, uriparts=uriparts, timeout=timeout, gzip=False)
+            secure=secure, uriparts=uriparts, timeout=timeout, gzip=False,
+            retry=False, verify_context=verify_context)
